from .. import Provider as PersonProvider


class Provider(PersonProvider):

    formats_male = (
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}-{{last_name}}',
        '{{prefix_male}} {{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}} {{suffix_male}}',
        '{{prefix_male}} {{first_name_male}} {{last_name}} {{suffix_male}}',
    )

    formats_female = (
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}-{{last_name}}',
        '{{prefix_female}} {{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}} {{suffix_female}}',
        '{{prefix_female}} {{first_name_female}} {{last_name}} {{suffix_female}}',
    )

    formats = formats_male + formats_female

    # source: https://www.data.gv.at/katalog/dataset/87fc82a0-0042-49c8-b6f9-2602cd3dc17a
    first_names_male = (
        'Aaron', 'Adam', 'Adrian', 'Adriano', 'Alan', 'Aleksander',
        'Alex', 'Alexandar', 'Alexander', 'Andreas', 'Andrej', 'Angelo', 'Anton',
        'Antonio', 'Antonius', 'Arda', 'Armin', 'Aron', 'Arthur', 'Aurelio', 'Axel',
        'Bastian', 'Ben', 'Benedict', 'Benedikt', 'Beni', 'Benjamin', 'Bernhard', 'Boris',
        'Bruno', 'Calvin', 'Carl', 'Carlo', 'Chris', 'Christian', 'Christoph',
        'Christopher', 'Clemens', 'Constantin', 'Cornelius', 'Cristiano', 'Damian', 'Daniel',
        'Danilo', 'Dario', 'Darius', 'Darko', 'David', 'Dennis', 'Dominik',
        'Eduard', 'Elias', 'Elyas', 'Emanuel', 'Emil', 'Emilian', 'Emmanuel', 'Eric',
        'Erik', 'Erwin', 'Fabian', 'Fabio', 'Felix', 'Ferdinand', 'Fernando', 'Filip',
        'Finn', 'Florentin', 'Florian', 'Florin', 'Franz', 'Frederik', 'Fridolin',
        'Friedrich', 'Gabriel', 'Georg', 'Gregor', 'Gustav', 'Heinrich', 'Henri',
        'Henrik', 'Henry', 'Hubert', 'Hugo', 'Igor', 'Ilias', 'Isa', 'Ismail', 'Jacob',
        'Jakob', 'James', 'Jamie', 'Jan', 'Jannik', 'Jannis', 'Jasper', 'Joel', 'Johann',
        'Johannes', 'John', 'Jonas', 'Jonathan', 'Josef', 'Joseph', 'Joshua', 'Julian',
        'Julius', 'Justin', 'Justus', 'Kai', 'Karim', 'Karl', 'Kevin', 'Kilian', 'Konrad',
        'Konstantin', 'Kristian', 'Lars', 'Laurenz', 'Laurin', 'Lean', 'Leander', 'Lennard',
        'Lennart', 'Leo', 'Leon', 'Leonard', 'Leonardo', 'Leonhard', 'Leopold', 'Levi',
        'Liam', 'Lino', 'Linus', 'Lionel', 'Lorenz', 'Lorenzo', 'Louis', 'Luca', 'Lucas',
        'Luis', 'Luka', 'Lukas', 'Maksim', 'Manuel', 'Marc', 'Marcel', 'Marco', 'Marcus',
        'Mario', 'Marius', 'Mark', 'Marko', 'Markus', 'Martin', 'Marvin',
        'Mateo', 'Matheo', 'Mathias', 'Matteo', 'Matthias', 'Maurice', 'Max',
        'Maximilian', 'Merlin', 'Mert', 'Michael', 'Mika', 'Mike', 'Milan', 'Milo', 'Moritz',
        'Natan', 'Nathan', 'Nicholas', 'Nick', 'Nico', 'Nicolai', 'Nicolas', 'Niklas',
        'Niko', 'Nikola', 'Nikolai', 'Nikolas', 'Nikolaus', 'Nils', 'Nino', 'Noah',
        'Noel', 'Oliver', 'Oscar', 'Oskar', 'Pascal', 'Patrick', 'Patrik', 'Paul',
        'Peter', 'Philip', 'Philipp', 'Phillip', 'Raffael', 'Ralph', 'Raphael',
        'Rene', 'Ricardo', 'Richard', 'Robert', 'Robin', 'Roman', 'Ruben', 'Sam',
        'Samuel', 'Sandro', 'Sascha', 'Sebastian', 'Severin', 'Simon', 'Stefan', 'Stephan',
        'Steven', 'Sven', 'Teodor', 'Theo', 'Theodor', 'Thomas', 'Tim', 'Timo', 'Timon',
        'Tobias', 'Tom', 'Tristan', 'Valentin', 'Valentino', 'Victor', 'Viktor',
        'Vincent', 'Vito', 'William', 'Xavier',
    )

    # source: https://www.data.gv.at/katalog/dataset/87fc82a0-0042-49c8-b6f9-2602cd3dc17a
    first_names_female = (
        'Adelina', 'Adriana', 'Ajna', 'Alara', 'Aleksandra', 'Alena', 'Alexa', 'Alexandra',
        'Alexia', 'Alice', 'Alma', 'Amanda', 'Amelia', 'Amelie', 'Anabella', 'Anastasia',
        'Andjela', 'Andjelina', 'Andrea', 'Angela', 'Angelika',
        'Angelina', 'Anika', 'Anita', 'Anja', 'Anna', 'Anna-Lena', 'Anna-Maria', 'Annabell',
        'Annabella', 'Annabelle', 'Annalena', 'Anne', 'Annika', 'Antonella', 'Antonia',
        'Ariana', 'Ariane', 'Aurelia', 'Aurora', 'Ava', 'Aylin', 'Barbara', 'Beatrice',
        'Bernadette', 'Berra', 'Bianca', 'Carina', 'Carla', 'Carlotta', 'Carolina',
        'Caroline', 'Catharina', 'Cecilia', 'Charlotte', 'Christina', 'Christine', 'Claire',
        'Clara', 'Clarissa', 'Claudia', 'Constanze', 'Cristina', 'Dana', 'Daniela', 'Denise',
        'Diana', 'Dilara', 'Domenica', 'Dora', 'Eda', 'Edda', 'Ela', 'Elena', 'Eleonora',
        'Elina', 'Elisa', 'Elisabeth', 'Ella', 'Ellie', 'Elma', 'Elona', 'Elsa', 'Elvira',
        'Emanuela', 'Emely', 'Emilia', 'Emilie', 'Emilija', 'Emma', 'Erina', 'Estelle',
        'Esther', 'Eva', 'Evelyn', 'Felicitas', 'Fiona', 'Florentina', 'Francesca', 'Franziska',
        'Frida', 'Gabriela', 'Gloria', 'Hanna', 'Hannah', 'Heidi', 'Helena', 'Helene', 'Ina',
        'Ines', 'Irina', 'Iris', 'Irma', 'Isabel', 'Isabell', 'Isabella',
        'Isabelle', 'Jana', 'Janine', 'Jasmina', 'Jasmine', 'Jennifer', 'Jessica',
        'Johanna', 'Josefine', 'Jovana', 'Julia', 'Juliana', 'Juliane',
        'Julijana', 'Juna', 'Kalina', 'Karina', 'Karla', 'Karolina', 'Karoline',
        'Katarina', 'Katharina', 'Katja', 'Kerstin', 'Klara', 'Kristina',
        'Kyra', 'Laetitia', 'Laila', 'Lana', 'Lara', 'Lara-Sophie', 'Larissa', 'Laura',
        'Laureen', 'Lea', 'Lea-Sophie', 'Leah', 'Leandra', 'Lena', 'Leni', 'Leona', 'Leoni',
        'Leonie', 'Leonora', 'Leontina', 'Leticia', 'Leyla', 'Lia', 'Lilia', 'Lilian',
        'Liliana', 'Liliane', 'Lilli', 'Lilly', 'Lily', 'Lina', 'Linda', 'Linnea', 'Lisa',
        'Lisa-Marie', 'Lola', 'Lora', 'Lorena', 'Lotta', 'Lotte', 'Louisa',
        'Louise', 'Luana', 'Lucia', 'Lucie', 'Luisa', 'Luise', 'Luna', 'Lydia',
        'Madeleine', 'Magdalena', 'Maida', 'Maja', 'Malena', 'Manuela', 'Mara', 'Maria',
        'Mariam', 'Mariana', 'Marie', 'Marie-Louise', 'Marie-Sophie', 'Mariella',
        'Marijana', 'Marina', 'Marissa', 'Marlene', 'Marta', 'Martha', 'Martina', 'Maryam',
        'Mathilda', 'Matilda', 'Maya', 'Melanie', 'Melek', 'Melina', 'Melisa', 'Melissa',
        'Mia', 'Michaela', 'Michelle', 'Mila', 'Milica', 'Mina', 'Mira', 'Miriam',
        'Mona', 'Nadia', 'Nadin', 'Nadine', 'Nadja', 'Naomi', 'Natalia', 'Natalie',
        'Natascha', 'Nathalie', 'Nela', 'Nele', 'Nelly', 'Nicola', 'Nicole', 'Nika',
        'Nikita', 'Nikola', 'Nikolina', 'Nina', 'Nisa', 'Nora', 'Norah', 'Olivia',
        'Patricia', 'Paula', 'Paulina', 'Pauline', 'Petra', 'Philippa', 'Pia', 'Rachel',
        'Raffaela', 'Rana', 'Rayana', 'Rebecca', 'Rita', 'Romy', 'Ronja', 'Ronya', 'Rosa',
        'Rosalie', 'Ruth', 'Sabine', 'Sabrina', 'Sahra', 'Salma',
        'Sandra', 'Sara', 'Sarah', 'Selena', 'Selin', 'Selina', 'Selma', 'Sena', 'Siena',
        'Sigrid', 'Sofia', 'Sofie', 'Sofija', 'Sonja', 'Sophia', 'Sophie', 'Sophie-Marie',
        'Soraya', 'Stefanie', 'Stella', 'Stephanie', 'Tamara', 'Tanja',
        'Tea', 'Theodora', 'Theresa', 'Therese', 'Tiffany', 'Tina',
        'Valentina', 'Vanessa', 'Vera', 'Verena', 'Veronika', 'Victoria',
        'Viktoria', 'Viola', 'Violetta', 'Vivian',
        'Yasmina', 'Ylvie', 'Yvonne', 'Zara', 'Zoe', 'Zoey',
    )

    first_names = first_names_male + first_names_female

    # about 1000 of the most popular Austrian surnames
    # https://de.wiktionary.org/wiki/Verzeichnis:Deutsch/Namen/die_h%C3%A4ufigsten_Nachnamen_%C3%96sterreichs
    last_names = (
        'Achleitner', 'Ackerl', 'Adam', 'Adler', 'Aichholzer', 'Aichinger',
        'Aigner', 'Albrecht', 'Altmann', 'Amann', 'Amon', 'Anderl', 'Angerer', 'Arnold',
        'Artner', 'Aschauer', 'Auer', 'Augustin', 'Auinger', 'Bacher', 'Bachinger',
        'Bachler', 'Bachmann', 'Bader', 'Baier', 'Baldauf', 'Barth', 'Bartl', 'Bauer',
        'Baumann', 'Baumgartner', 'Bayer', 'Beck', 'Becker', 'Beer', 'Berchtold', 'Berger',
        'Bergmann', 'Berner', 'Bernhard', 'Berthold', 'Bichler', 'Biedermann', 'Binder',
        'Bischof', 'Bitschnau', 'Bittner', 'Blauensteiner', 'Blum', 'Blümel', 'Bock',
        'Bodner', 'Bogner', 'Brandl', 'Brandner', 'Brandstetter', 'Brandstätter',
        'Brandtner', 'Braun', 'Brenner', 'Breuer', 'Bruckner', 'Brugger', 'Brunner', 'Bräuer',
        'Buchberger', 'Buchegger', 'Bucher', 'Buchinger', 'Buchner', 'Burger', 'Burgstaller',
        'Burtscher', 'Bäck', 'Böck', 'Böhler', 'Böhm', 'Bösch', 'Bürger', 'Dallinger',
        'Dangl', 'Danner', 'Danninger', 'Decker', 'Dengg', 'Denk', 'Deutschmann', 'Dietl',
        'Dietrich', 'Dirnberger', 'Dittrich', 'Dobler', 'Doppler', 'Dorfer', 'Dorn',
        'Dorner', 'Draxler', 'Dünser', 'Eberhard', 'Eberharter', 'Eberl', 'Ebner', 'Ecker',
        'Eder', 'Edlinger', 'Egger', 'Eibl', 'Eichberger', 'Eichhorn', 'Eichinger',
        'Eisl', 'Eisner', 'Eller', 'Ender', 'Engel', 'Engl', 'Enzinger', 'Erber',
        'Erhart', 'Erlacher', 'Erler', 'Ernst', 'Ertl', 'Fabian', 'Falkner', 'Fankhauser',
        'Farkas', 'Fasching', 'Fehringer', 'Feichtenschlager', 'Feichter', 'Feichtinger',
        'Feichtner', 'Feigl', 'Felber', 'Felder', 'Fellinger', 'Fellner', 'Fercher', 'Ferstl',
        'Fichtinger', 'Fiedler', 'Fink', 'Fischer', 'Fitz', 'Fleck', 'Fleischhacker',
        'Fleischmann', 'Foidl', 'Forster', 'Forstner', 'Frank', 'Franz', 'Freitag',
        'Freudenthaler', 'Frey', 'Frick', 'Friedl', 'Friedrich', 'Frisch', 'Fritsch', 'Fritz',
        'Froschauer', 'Fröhlich', 'Fröschl', 'Frühwirth', 'Fuchs', 'Fuhrmann', 'Füreder',
        'Fürst', 'Gabriel', 'Gahleitner', 'Galler', 'Gamsjäger', 'Gangl', 'Gartner',
        'Gasser', 'Gassner', 'Gattringer', 'Geier', 'Geiger', 'Geisler', 'Geyer', 'Gindl',
        'Glaser', 'Glatz', 'Glück', 'Gmeiner', 'Gollner', 'Gosch', 'Grabher', 'Grabner',
        'Graf', 'Grasser', 'Grassl', 'Gratz', 'Gratzer', 'Gratzl', 'Greiner', 'Griesser',
        'Grill', 'Gritsch', 'Gross', 'Groß', 'Gruber', 'Grundner', 'Grünberger', 'Grüner',
        'Grünwald', 'Gschaider', 'Gschwandtner', 'Gstrein', 'Guggenberger', 'Gutmann',
        'Gärtner', 'Göschl', 'Götz', 'Günther', 'Haas', 'Haberl', 'Hacker', 'Hackl',
        'Haderer', 'Hafner', 'Hagen', 'Hager', 'Hahn', 'Haid', 'Haiden', 'Haider',
        'Haidinger', 'Haindl', 'Hainzl', 'Haller', 'Hammer', 'Hammerer', 'Hammerl', 'Handl',
        'Handler', 'Haring', 'Harrer', 'Hartl', 'Hartmann', 'Haslauer', 'Haslinger',
        'Hattinger', 'Hauer', 'Haumer', 'Hausberger', 'Hauser', 'Hebenstreit', 'Hechenberger',
        'Heger', 'Heigl', 'Heim', 'Heindl', 'Heinrich', 'Heinz', 'Heinzl', 'Heiss',
        'Heissenberger', 'Held', 'Hell', 'Heller', 'Helm', 'Hemetsberger', 'Herbst', 'Hermann',
        'Herrmann', 'Herzog', 'Himmelbauer', 'Hinterberger', 'Hinteregger', 'Hinterleitner',
        'Hirsch', 'Hirschmann', 'Hochleitner', 'Hochreiter', 'Hofbauer', 'Hofer',
        'Hoffmann', 'Hofinger', 'Hofmann', 'Hofmeister', 'Hofstetter', 'Hofstätter', 'Holl',
        'Hollaus', 'Holler', 'Holzer', 'Holzinger', 'Holzknecht', 'Holzmann', 'Horak',
        'Horn', 'Hosp', 'Huber', 'Hubmann', 'Huemer', 'Hufnagl', 'Humer', 'Hummel',
        'Hummer', 'Huter', 'Hutter', 'Hutterer', 'Hämmerle', 'Häusler', 'Hödl', 'Höfer',
        'Höfler', 'Höglinger', 'Höller', 'Hölzl', 'Hörl', 'Hörmann', 'Hübner', 'Hütter',
        'Jahn', 'Jandl', 'Janisch', 'Jank', 'Jauk', 'Jenewein', 'Jost', 'Jovanovic',
        'Juen', 'Jung', 'Jungwirth', 'Jäger', 'Jöbstl', 'Kager', 'Kahr', 'Kain',
        'Kaindl', 'Kainz', 'Kaiser', 'Kalcher', 'Kaltenbrunner', 'Kaltenböck',
        'Kaltenegger', 'Kammerer', 'Kapeller', 'Kappel', 'Kargl', 'Karl', 'Karner', 'Karrer',
        'Kaspar', 'Kasper', 'Kastner', 'Kaufmann', 'Keller', 'Kellner', 'Keplinger',
        'Kern', 'Kerschbaum', 'Kerschbaumer', 'Kessler', 'Kirchmair', 'Kirchner',
        'Kirschner', 'Kiss', 'Kitzler', 'Klammer', 'Klaus', 'Klausner', 'Klein', 'Klement',
        'Klinger', 'Klingler', 'Klocker', 'Kloiber', 'Klotz', 'Klug', 'Knapp', 'Knaus',
        'Knoll', 'Kober', 'Koch', 'Kocher', 'Kofler', 'Kogler', 'Kohl', 'Kohler', 'Kolar',
        'Kolb', 'Koller', 'Kollmann', 'Kolm', 'Konrad', 'Kopf', 'Kopp', 'Koppensteiner',
        'Kraft', 'Krainer', 'Krainz', 'Kral', 'Krall', 'Kramer', 'Krammer', 'Kratzer',
        'Kraus', 'Kraxner', 'Kreidl', 'Kreiner', 'Kremser', 'Krenn', 'Kreuzer', 'Kriegl',
        'Kronberger', 'Kronsteiner', 'Krug', 'Kröll', 'Kucera', 'Kugler', 'Kuhn', 'Kummer',
        'Kunz', 'Kurz', 'Kurzmann', 'Käfer', 'Köberl', 'Köck', 'Köhler', 'Kölbl', 'Köll',
        'König', 'Kössler', 'Lackner', 'Ladner', 'Lagler', 'Laimer', 'Lammer', 'Lampert',
        'Lampl', 'Lamprecht', 'Landl', 'Lang', 'Langer', 'Larcher', 'Lassnig', 'Leber',
        'Lechner', 'Lederer', 'Leeb', 'Lehner', 'Leibetseder', 'Leitgeb', 'Leithner',
        'Leitner', 'Lengauer', 'Lenz', 'Leonhartsberger', 'Leopold', 'Lerch', 'Lercher',
        'Lettner', 'Leutgeb', 'Lichtenegger', 'Linder', 'Lindinger', 'Lindner', 'Lindorfer',
        'Lintner', 'Lipp', 'List', 'Loibl', 'Loidl', 'Lorenz', 'Ludwig', 'Luger',
        'Luttenberger', 'Lutz', 'Löffler', 'Macher', 'Mader', 'Maier', 'Maierhofer', 'Mair',
        'Mairhofer', 'Mandl', 'Mann', 'Margreiter', 'Maringer', 'Mark', 'Markl', 'Marte',
        'Martin', 'Marx', 'Mathis', 'Maurer', 'Mayer', 'Mayerhofer', 'Mayr', 'Mayrhofer',
        'Meier', 'Meindl', 'Meister', 'Meixner', 'Messner', 'Metzler', 'Meusburger',
        'Meyer', 'Mitter', 'Mitteregger', 'Mitterer', 'Mitterlehner', 'Mittermayr',
        'Mohr', 'Moosbrugger', 'Moritz', 'Moser', 'Muhr', 'Mörth', 'Mühlbacher',
        'Mühlberger', 'Mühlböck', 'Müller', 'Müllner', 'Nagel', 'Nagele', 'Nagl', 'Nemeth',
        'Neubacher', 'Neubauer', 'Neugebauer', 'Neuhauser', 'Neuhold', 'Neulinger', 'Neumann',
        'Neumayer', 'Neumayr', 'Neumeister', 'Neumüller', 'Neuner', 'Neureiter', 'Neuwirth',
        'Niederl', 'Nowak', 'Nussbaumer', 'Nußbaumer', 'Nöbauer', 'Oberhauser', 'Oberhofer',
        'Oberleitner', 'Obermayr', 'Obermüller', 'Oberndorfer', 'Ofner', 'Ortner', 'Ostermann',
        'Oswald', 'Ott', 'Pacher', 'Pachler', 'Paier', 'Pammer', 'Parzer', 'Pauer', 'Paul',
        'Paulitsch', 'Payer', 'Peer', 'Peham', 'Pendl', 'Penz', 'Perner', 'Pertl',
        'Pesendorfer', 'Peter', 'Petz', 'Pfeffer', 'Pfeifer', 'Pfeiffer', 'Pfister', 'Pfleger',
        'Philipp', 'Pichler', 'Pieber', 'Pilz', 'Pinter', 'Pircher', 'Pirker', 'Plank',
        'Plattner', 'Platzer', 'Pock', 'Pohl', 'Pointner', 'Pokorny', 'Pollak', 'Polzer',
        'Posch', 'Postl', 'Prager', 'Prantl', 'Praxmarer', 'Prem', 'Prenner', 'Prinz',
        'Probst', 'Prohaska', 'Pröll', 'Pucher', 'Puchner', 'Puntigam', 'Punz', 'Putz',
        'Pöll', 'Pölzl', 'Pöschl', 'Pühringer', 'Raab', 'Rabitsch', 'Rabl', 'Radl',
        'Rainer', 'Ramsauer', 'Rath', 'Rauch', 'Rausch', 'Rauscher', 'Rauter',
        'Rechberger', 'Redl', 'Reich', 'Reichel', 'Reicher', 'Reichl', 'Reichmann', 'Reif',
        'Reinbacher', 'Reindl', 'Reiner', 'Reinisch', 'Reinprecht', 'Reinthaler', 'Reischl',
        'Reisinger', 'Reisner', 'Reitbauer', 'Reiter', 'Reiterer', 'Reithofer', 'Reitinger',
        'Renner', 'Resch', 'Rettenbacher', 'Richter', 'Rieder', 'Riedl', 'Riedler',
        'Riedmann', 'Rieger', 'Riegler', 'Riener', 'Riepl', 'Rieser', 'Ringhofer', 'Rinner',
        'Ritter', 'Rohrer', 'Rohrmoser', 'Rosenberger', 'Rosner', 'Rossmann', 'Roth',
        'Rottensteiner', 'Rotter', 'Rudolf', 'Rupp', 'Röck', 'Rössler', 'Sagmeister', 'Sailer',
        'Salcher', 'Salzer', 'Salzmann', 'Sammer', 'Santner', 'Sattler', 'Sauer',
        'Schachinger', 'Schachner', 'Schaffer', 'Schalk', 'Schaller', 'Schandl', 'Schantl',
        'Scharf', 'Scharinger', 'Schartner', 'Schatz', 'Schatzl', 'Schauer', 'Scheer',
        'Scheiber', 'Scheidl', 'Schenk', 'Scherer', 'Scherr', 'Scherz', 'Scherzer',
        'Scheucher', 'Schiefer', 'Schiestl', 'Schilcher', 'Schiller', 'Schimpl', 'Schinagl',
        'Schindler', 'Schinnerl', 'Schlager', 'Schlosser', 'Schlögl', 'Schmid', 'Schmidinger',
        'Schmidl', 'Schmidt', 'Schmied', 'Schmuck', 'Schmölzer', 'Schnabl', 'Schneeberger',
        'Schneider', 'Schober', 'Scholz', 'Schramm', 'Schrammel', 'Schranz', 'Schreiber',
        'Schreiner', 'Schrempf', 'Schrenk', 'Schrittwieser', 'Schröder', 'Schubert', 'Schuh',
        'Schuler', 'Schuller', 'Schulz', 'Schuster', 'Schwab', 'Schwaiger', 'Schwaighofer',
        'Schwarz', 'Schwarzinger', 'Schwarzl', 'Schweiger', 'Schweighofer', 'Schweitzer',
        'Schwendinger', 'Schäfer', 'Schöberl', 'Schöffmann', 'Schöller', 'Schön', 'Schönauer',
        'Schönberger', 'Schöpf', 'Schüller', 'Schütz', 'Seebacher', 'Seidl', 'Seifert',
        'Seiler', 'Seiser', 'Seitz', 'Seiwald', 'Sieber', 'Sieberer', 'Siegl', 'Sigl',
        'Siller', 'Simic', 'Simon', 'Singer', 'Sommer', 'Sonnleitner', 'Sorger', 'Sperl',
        'Spiegl', 'Spindler', 'Spitzer', 'Spreitzer', 'Springer', 'Stadlbauer', 'Stadler',
        'Stangl', 'Stark', 'Staudacher', 'Staudinger', 'Stecher', 'Stefan', 'Steger',
        'Steidl', 'Steiger', 'Steinacher', 'Steinbacher', 'Steinbauer', 'Steinberger',
        'Steinböck', 'Steindl', 'Steiner', 'Steininger', 'Steinkellner', 'Steinlechner',
        'Steinwender', 'Stelzer', 'Stelzl', 'Stern', 'Steurer', 'Stiegler', 'Stifter', 'Stock',
        'Stocker', 'Stockhammer', 'Stockinger', 'Stoiber', 'Stolz', 'Strasser', 'Strauss',
        'Strauß', 'Streicher', 'Strobl', 'Strohmaier', 'Strohmayer', 'Strohmeier',
        'Stummer', 'Sturm', 'Stöckl', 'Stöger', 'Stückler', 'Stütz', 'Sulzer', 'Suppan',
        'Taferner', 'Tanzer', 'Tauber', 'Taucher', 'Teufl', 'Thaler', 'Thalhammer',
        'Thaller', 'Thurner', 'Tiefenbacher', 'Tischler', 'Toth', 'Trattner', 'Trauner',
        'Traxler', 'Trimmel', 'Trinkl', 'Trummer', 'Uhl', 'Ullmann', 'Ulrich', 'Unger',
        'Unterberger', 'Unterweger', 'Urban', 'Varga', 'Veit', 'Vogel', 'Vogl', 'Vogler',
        'Vogt', 'Wachter', 'Wagner', 'Walch', 'Walcher', 'Walder', 'Waldner', 'Wallner',
        'Walser', 'Walter', 'Waltl', 'Wandl', 'Weber', 'Wechselberger', 'Wegscheider',
        'Weidinger', 'Weigl', 'Weinberger', 'Weiser', 'Weiss', 'Weissenböck', 'Weiß',
        'Wenger', 'Weninger', 'Wenzl', 'Werner', 'Widhalm', 'Widmann', 'Wiedner',
        'Wieland', 'Wiener', 'Wiesbauer', 'Wieser', 'Wiesinger', 'Wiesner', 'Wild',
        'Wilfinger', 'Wilhelm', 'Wimmer', 'Windhager', 'Windisch', 'Winkler', 'Winter',
        'Wirth', 'Wittmann', 'Wohlmuth', 'Wolf', 'Wurm', 'Wurzer', 'Wurzinger',
        'Wögerbauer', 'Wöhrer', 'Yilmaz', 'Zach', 'Zangerl', 'Zauner', 'Zechmeister',
        'Zechner', 'Zehetner', 'Zeiler', 'Zeilinger', 'Zeiner', 'Zeller', 'Zenz', 'Zettl',
        'Ziegler', 'Zimmermann', 'Zotter', 'Zöchling', 'Zöhrer',
    )

    prefixes_male = (
        'Herr', 'Dr.', 'Ing.', 'Dipl.-Ing.', 'Prof.', 'Univ.Prof.',
    )
    prefixes_female = (
        'Frau', 'Dr.', 'Ing.', 'Dipl.-Ing.', 'Prof.', 'Univ.Prof.',
    )

    prefixes_male = (
        'Herr', 'Dr.', 'Ing.', 'Dipl.-Ing.', 'Prof.', 'Univ.Prof.',
    )
    prefixes_female = (
        'Frau', 'Dr.', 'Ing.', 'Dipl.-Ing.', 'Prof.', 'Univ.Prof.',
    )

    prefixes = ('Dr.', 'Mag.', 'Ing.', 'Dipl.-Ing.', 'Prof.', 'Univ.Prof.')



from .. import Provider as BaseProvider


class Provider(BaseProvider):
    # jobs parsed from a list provided by State Treasury:
    # http://www.valtiokonttori.fi/download/noname/%7BF69EA5BD-C919-49FE-8D51-91434E4B030D%7D/82158
    jobs = ["Agrologi",
            "Aikuiskoulutusjohtaja",
            "Aineenopettaja",
            "Ajojärjestelijä",
            "Akatemian tutkijatohtori",
            "Aktuaari",
            "Alakoulujen apulaisrehtori",
            "Alikersantti",
            "Alkoholiasiain ylitarkastaja",
            "Back office asiantuntija",
            "Ballistikko",
            "Bioanalyytikko",
            "Brand manager",
            "Budjettiassistentti",
            "Business controller",
            "Cc-yritysneuvoja",
            "Cert-fi -yksikön päällikkö",
            "Communication officer",
            "Consul",
            "Counsellor",
            "Data-analyytikko",
            "Dekaanin sihteeri",
            "Dieettikeittäjä",
            "Digitaalisen kokeen toteuttaja",
            "Diplomi-insinööri",
            "Dokumentoija",
            "Dosentti",
            "Eakr-koordinaattori",
            "Editoija",
            "Edunvalvontasihteeri",
            "Egr-ohjaaja",
            "Ekokampuskoordinaattori",
            "Elektroniikka-asentaja",
            "Elinkeinopäällikkö",
            "Elokuvakonemestari",
            "Elputeknikko",
            "Eläinlääkintöneuvos",
            "Faktori",
            "Farmakologi",
            "Fidipro-professori",
            "Filmiteknikko",
            "Financial controller",
            "Floristi",
            "Fysioterapeutti",
            "Fyysikko",
            "Gemmologi",
            "Gentax-järjestelmäasiantuntija",
            "Geofyysikko",
            "Gis-asiantuntija",
            "Gm huto pääkäyttäjä",
            "Graafikko",
            "Haastattelija",
            "Hakukoordinaattori",
            "Hallimestari",
            "Hammashoitaja",
            "Hankearkkitehti",
            "Harjaantumisopetuksen erityisluokanopettaja",
            "Havainnontarkastaja",
            "Helikopterihuoltoaliupseeri",
            "Henkikirjoittaja",
            "Johtava kuluttajaoikeusneuvoja",
            "Ict-arkkitehti",
            "Ihmisoikeuskeskuksen johtaja",
            "Iktyonomi",
            "Ilma-aluksen päällikkö",
            "Iltapäiväkerhon ohjaaja",
            "Immunologi",
            "Info-palvelupisteen hoitaja",
            "Innoittaja",
            "Jakeluvastaava",
            "Jalkaväen tarkastaja",
            "Jaoksen johtaja",
            "Jatkokoulutettava eläinlääkäri",
            "Jhs-projektipäällikkö",
            "Johdon asiantuntija",
            "Joukkoliikenneasiantuntija",
            "Julkaisu- ja markkinointisuunnittelija",
            "Junamies",
            "Juontaja",
            "Kaapeli-insinööri",
            "Kabinettisihteeri",
            "Kadettikoulun johtaja",
            "Kahvila-apulainen",
            "Kairaaja",
            "Kalabiologi",
            "Kampanjapäällikkö",
            "Kanavanhoitaja",
            "Kapellimestari",
            "Karjamestari",
            "Laadunvarmistuksen asiantuntija",
            "Laboraattori",
            "Laillisuusvalvontasihteeri",
            "Laki- ja henkilöstöasiainjohtaja",
            "Lapsiasiavaltuutettu",
            "Laskennan kehittämispäällikkö",
            "Lataamoinsinööri",
            "Lautakuntasihteeri",
            "Lavastaja",
            "Maa- ja vesirakennustyöntekijä",
            "Maisema-arkkitehti",
            "Majakkateknikko",
            "Maksatusasiantuntija",
            "Malli",
            "Mareografihoitaja",
            "Mastoteknikko",
            "Matemaatikko",
            "Media- ja kulttuurikoordinaattori",
            "Neuropsykologi",
            "Nimikkeistöpäällikkö",
            "Nosturinkuljettaja",
            "Notaari",
            "Nukutuslääkäri",
            "Numerointisihteeri",
            "Nuorempi konstaapeli",
            "Näytearkistonhoitaja",
            "Näönkäytön asiantuntija",
            "Obduktiokoordinaattori",
            "Observaattori",
            "Offset-monistaja",
            "Ohjaaja",
            "Oikaisulautakunnan puheenjohtaja",
            "Oleskelulupakäsittelijä",
            "Omistajaohjausyksikön johtaja",
            "Ompelija",
            "Opas",
            "Operaatiopäällikkö",
            "Padonhoitaja",
            "Paikallisjohtaja",
            "Pakolaiskeskuksen johtaja",
            "Palkanlaskentapäällikkö",
            "Panostaja",
            "Paperikonservaattori",
            "Parturi-kampaaja",
            "Passi- ja maahantulolupavirkailija/toimistovirkailija",
            "Pataljoonan komentaja",
            "Pedagogi",
            "Radioasentaja",
            "Rahakammion johtaja",
            "Raideliikennejohtaja",
            "Rajaeläinlääkäri",
            "Rakennemuutosjohtaja",
            "Raportoinnin asiantuntija",
            "Ratainsinööri",
            "Rauhanturvaaja",
            "Ravintohaastattelija",
            "Rehtori",
            "Saamelaisarkistonhoitaja",
            "Sadehavainnontekijä",
            "Sairaala-apulainen",
            "Saksan, englannin ja ruotsinkielen lehtori",
            "Salkunhoitaja",
            "Sanomakeskusaliupseeri",
            "Satamapäällikkö",
            "Seismologi",
            "Sektorijohtaja",
            "Selvittelijä",
            "Taajuussuunnittelija",
            "Taideamanuenssi",
            "Tallentaja",
            "Tanssija",
            "Tapahtumakoordinaattori",
            "Tarjoilija",
            "Tasa-arvoneuvos",
            "Tavaraliikennelupakäsittelijä",
            "Team finland kasvu- ja kansainvälistymiskoordinaattori",
            "Teemapäällikkö",
            "Ulkoasiainneuvos",
            "Ulosottojohtaja",
            "Ultraäänihoitaja",
            "Unix-asiantuntija",
            "Upseeri",
            "Urakonsultti",
            "Urheiluohjaaja",
            "Vaaitsija",
            "Vac-yhdyshenkilö",
            "Vahingonkorvausasiantuntija",
            "Vaihteenhoitaja",
            "Vakuustoimittaja",
            "Valaistusmestari",
            "Vammaisasiamies",
            "Vanhempi tutkijainsinööri",
            "Vapaa-ajan ohjaaja",
            "Varadekaani",
            "Www-asiantuntija",
            "Yhdenvertaisuusvaltuutettu",
            "Yhteinen tuntiopettaja",
            "Yksikkösihteeri",
            "Yleinen edunvalvoja",
            "Yliaktuaari",
            "Ylläpidon palvelupäällikkö",
            "Yläasteen rehtori",
            "Ympärintönsuojeluyksikön päällikkö",
            "Yrittäjyysneuvoja",
            "Yva-koordinaattori"]



from .. import Provider as BaseProvider


class Provider(BaseProvider):
    """
    A Faker provider for the Romanian VAT IDs
    """

    vat_id_formats = (
        'RO##',
        'RO###',
        'RO####',
        'RO#####',
        'RO######',
        'RO#######',
        'RO########',
        'RO#########',
        'RO##########',
    )

    def vat_id(self):
        """
        http://ec.europa.eu/taxation_customs/vies/faq.html#item_11
        :return: A random Romanian VAT ID
        """

        return self.bothify(self.random_element(self.vat_id_formats))



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    # Currently this is my own work
    license_formats = (
        '? ### ??',
        '? ### ???',
        '?? ### ??',
        '?? ### ???',
        '? #### ??',
        '? #### ???',
        '?? #### ??',
        '?? #### ???',
    )



import re

from .. import BaseProvider


class Provider(BaseProvider):

    upc_e_base_pattern = re.compile(r'^\d{6}$')
    upc_ae_pattern1 = re.compile(
        r'^(?P<number_system_digit>[01])'   # The first digit must be 0 or 1
        r'(?=\d{11}$)'                      # followed by 11 digits of which
        r'(?P<mfr_code>\d{2})'              # the first 2 digits make up the manufacturer code,
        r'(?:(?P<extra>[012])0{4})'         # if immediately followed by 00000, 10000, or 20000,
        r'(?P<product_code>\d{3})'          # a 3-digit product code,
        r'(?P<check_digit>\d)$',            # and finally a check digit.
    )
    upc_ae_pattern2 = re.compile(
        r'^(?P<number_system_digit>[01])'   # The first digit must be 0 or 1
        r'(?=\d{11}$)'                      # followed by 11 digits of which
        r'(?P<mfr_code>\d{3,4}?)'           # the first 3 or 4 digits make up the manufacturer code,
        r'(?:0{5})'                         # if immediately followed by 00000,
        r'(?P<product_code>\d{1,2})'        # a 2-digit or single digit product code,
        r'(?P<check_digit>\d)$',            # and finally a check digit.
    )
    upc_ae_pattern3 = re.compile(
        r'^(?P<number_system_digit>[01])'   # The first digit must be 0 or 1
        r'(?=\d{11}$)'                      # followed by 11 digits of which
        r'(?P<mfr_code>\d{5})'              # the first 5 digits make up the manufacturer code,
        r'(?:0{4}(?P<extra>[5-9]))'         # if immediately followed by 0000 and a 5, 6, 7, 8, or 9,
        r'(?P<check_digit>\d)$',            # and finally a check digit.
    )

    def _ean(self, length=13, leading_zero=None):
        if length not in (8, 13):
            raise AssertionError("length can only be 8 or 13")

        code = [self.random_digit() for _ in range(length - 1)]
        if leading_zero is True:
            code[0] = 0
        elif leading_zero is False:
            code[0] = self.random_int(1, 9)

        if length == 8:
            weights = [3, 1, 3, 1, 3, 1, 3]
        elif length == 13:
            weights = [1, 3, 1, 3, 1, 3, 1, 3, 1, 3, 1, 3]

        weighted_sum = sum(x * y for x, y in zip(code, weights))
        check_digit = (10 - weighted_sum % 10) % 10
        code.append(check_digit)

        return ''.join(str(x) for x in code)

    def _convert_upc_a2e(self, upc_a):
        """
        Convert a 12-digit UPC-A barcode to its 8-digit UPC-E equivalent.

        Note that not all UPC-A barcodes can be converted.
        """
        if not isinstance(upc_a, str):
            raise TypeError('`upc_a` is not a string')
        m1 = self.upc_ae_pattern1.match(upc_a)
        m2 = self.upc_ae_pattern2.match(upc_a)
        m3 = self.upc_ae_pattern3.match(upc_a)
        if not any([m1, m2, m3]):
            raise ValueError('`upc_a` has an invalid value')
        upc_e_template = '{number_system_digit}{mfr_code}{product_code}{extra}{check_digit}'
        if m1:
            upc_e = upc_e_template.format(**m1.groupdict())
        elif m2:
            groupdict = m2.groupdict()
            groupdict['extra'] = str(len(groupdict.get('mfr_code')))
            upc_e = upc_e_template.format(**groupdict)
        else:
            groupdict = m3.groupdict()
            groupdict['product_code'] = ''
            upc_e = upc_e_template.format(**groupdict)
        return upc_e

    def _upc_ae(self, base=None, number_system_digit=None):
        """
        Create a 12-digit UPC-A barcode that can be converted to UPC-E.

        The expected value of ``base`` is a 6-digit string. If any other value is
        provided, this method will use a random 6-digit string instead.

        The expected value of ``number_system_digit`` is the integer ``0`` or ``1``.
        If any other value is provided, this method will randomly choose from the two.

        Please also view notes on `upc_a()` and `upc_e()` for more details.
        """
        if isinstance(base, str) and self.upc_e_base_pattern.match(base):
            base = [int(x) for x in base]
        else:
            base = [self.random_int(0, 9) for _ in range(6)]
        if number_system_digit not in [0, 1]:
            number_system_digit = self.random_int(0, 1)

        if base[-1] <= 2:
            code = base[:2] + base[-1:] + [0] * 4 + base[2:-1]
        elif base[-1] <= 4:
            code = base[:base[-1]] + [0] * 5 + base[base[-1]:-1]
        else:
            code = base[:5] + [0] * 4 + base[-1:]

        code.insert(0, number_system_digit)
        weights = [3, 1, 3, 1, 3, 1, 3, 1, 3, 1, 3]
        weighted_sum = sum(x * y for x, y in zip(code, weights))
        check_digit = (10 - weighted_sum % 10) % 10
        code.append(check_digit)
        return ''.join(str(x) for x in code)

    def ean(self, length=13):
        """Generate an EAN barcode of the specified ``length``.

        The value of ``length`` can only be ``8`` or ``13`` (default)  which will
        create an EAN-8 or an EAN-13 barcode respectively.

        :sample: length=13
        :sample: length=8
        """
        return self._ean(length)

    def ean8(self):
        """Generate an EAN-8 barcode.

        This method uses :meth:`ean() <faker.providers.barcode.Provider.ean>` under the
        hood with the ``length`` argument explicitly set to ``8``.

        :sample:
        """
        return self._ean(8)

    def ean13(self, leading_zero=None):
        """Generate an EAN-13 barcode.

        If ``leading_digit`` is ``True``, the leftmost digit of the barcode will be set
        to ``0``. If ``False``, the leftmost digit cannot be ``0``. If ``None`` (default),
        the leftmost digit can be any digit.

        Note that an EAN-13 barcode that starts with a zero can be converted to UPC-A
        by dropping the leading zero.

        This method uses :meth:`ean() <faker.providers.barcode.Provider.ean>` under the
        hood with the ``length`` argument explicitly set to ``13``.

        :sample:
        :sample: leading_zero=False
        :sample: leading_zero=True
        """
        return self._ean(13, leading_zero=leading_zero)

    def upc_a(self, upc_ae_mode=False, base=None, number_system_digit=None):
        """Generate a 12-digit UPC-A barcode.

        The value of ``upc_ae_mode`` controls how barcodes will be generated. If ``False``
        (default), barcodes are not guaranteed to have a UPC-E equivalent. In this mode,
        the method uses :meth:`ean13 <faker.providers.barcode.Provider.ean13>` under the hood,
        and the values of ``base`` and ``number_system_digit`` will be ignored.

        If ``upc_ae_mode`` is ``True``, the resulting barcodes are guaranteed to have a UPC-E
        equivalent, and the values of ``base`` and ``number_system_digit`` will be used to
        control what is generated.

        Under this mode, ``base`` is expected to have a 6-digit string value. If any other value
        is supplied, a random 6-digit string will be used instead. As for ``number_system_digit``,
        the expected value is a ``0`` or a ``1``. If any other value is provided, this method
        will randomly choose from the two.

        .. important::

           When ``upc_ae_mode`` is enabled, you might encounter instances where different values
           of ``base`` (e.g. ``'120003'`` and ``'120004'``) produce the same UPC-A barcode. This
           is normal, and the reason lies within the whole conversion process. To learn more about
           this and what ``base`` and ``number_system_digit`` actually represent, please refer
           to :meth:`upc_e() <faker.providers.barcode.Provider.upc_e>`.

        :sample:
        :sample: upc_ae_mode=True, number_system_digit=0
        :sample: upc_ae_mode=True, number_system_digit=1
        :sample: upc_ae_mode=True, base='123456', number_system_digit=0
        :sample: upc_ae_mode=True, base='120003', number_system_digit=0
        :sample: upc_ae_mode=True, base='120004', number_system_digit=0
        """
        if upc_ae_mode is True:
            return self._upc_ae(base=base, number_system_digit=number_system_digit)
        else:
            ean13 = self.ean13(leading_zero=True)
            return ean13[1:]

    def upc_e(self, base=None, number_system_digit=None, safe_mode=True):
        """Generate an 8-digit UPC-E barcode.

        UPC-E barcodes can be expressed in 6, 7, or 8-digit formats, but this method uses the
        8 digit format, since it is trivial to convert to the other two formats. The first digit
        (starting from the left) is controlled by ``number_system_digit``, and it can only be a
        ``0`` or a ``1``. The last digit is the check digit that is inherited from the UPC-E barcode's
        UPC-A equivalent. The middle six digits are collectively referred to as the ``base`` (for a
        lack of a better term).

        On that note, this method uses ``base`` and ``number_system_digit`` to first generate a
        UPC-A barcode for the check digit, and what happens next depends on the value of ``safe_mode``.
        The argument ``safe_mode`` exists, because there are some UPC-E values that share the same
        UPC-A equivalent. For example, any UPC-E barcode of the form ``abc0000d``, ``abc0003d``, and
        ``abc0004d`` share the same UPC-A value ``abc00000000d``, but that UPC-A value will only convert
        to ``abc0000d`` because of (a) how UPC-E is just a zero-suppressed version of UPC-A and (b) the
        rules around the conversion.

        If ``safe_mode`` is ``True`` (default), this method performs another set of conversions to
        guarantee that the UPC-E barcodes generated can be converted to UPC-A, and that UPC-A
        barcode can be converted back to the original UPC-E barcode. Using the example above, even
        if the bases ``120003`` or ``120004`` are used, the resulting UPC-E barcode will always
        use the base ``120000``.

        If ``safe_mode`` is ``False``, then the ``number_system_digit``, ``base``, and the computed
        check digit will just be concatenated together to produce the UPC-E barcode, and attempting
        to convert the barcode to UPC-A and back again to UPC-E will exhibit the behavior described
        above.

        :sample:
        :sample: base='123456'
        :sample: base='123456', number_system_digit=0
        :sample: base='123456', number_system_digit=1
        :sample: base='120000', number_system_digit=0
        :sample: base='120003', number_system_digit=0
        :sample: base='120004', number_system_digit=0
        :sample: base='120000', number_system_digit=0, safe_mode=False
        :sample: base='120003', number_system_digit=0, safe_mode=False
        :sample: base='120004', number_system_digit=0, safe_mode=False
        """
        if safe_mode is not False:
            upc_ae = self._upc_ae(base=base, number_system_digit=number_system_digit)
            return self._convert_upc_a2e(upc_ae)
        else:
            upc_ae = self._upc_ae(base=base, number_system_digit=number_system_digit)
            return upc_ae[0] + ''.join(str(x) for x in base) + upc_ae[-1]



from collections import OrderedDict

from .. import BaseProvider
from .color import RandomColor

localized = True


class Provider(BaseProvider):
    all_colors = OrderedDict((
        ("AliceBlue", "#F0F8FF"),
        ("AntiqueWhite", "#FAEBD7"),
        ("Aqua", "#00FFFF"),
        ("Aquamarine", "#7FFFD4"),
        ("Azure", "#F0FFFF"),
        ("Beige", "#F5F5DC"),
        ("Bisque", "#FFE4C4"),
        ("Black", "#000000"),
        ("BlanchedAlmond", "#FFEBCD"),
        ("Blue", "#0000FF"),
        ("BlueViolet", "#8A2BE2"),
        ("Brown", "#A52A2A"),
        ("BurlyWood", "#DEB887"),
        ("CadetBlue", "#5F9EA0"),
        ("Chartreuse", "#7FFF00"),
        ("Chocolate", "#D2691E"),
        ("Coral", "#FF7F50"),
        ("CornflowerBlue", "#6495ED"),
        ("Cornsilk", "#FFF8DC"),
        ("Crimson", "#DC143C"),
        ("Cyan", "#00FFFF"),
        ("DarkBlue", "#00008B"),
        ("DarkCyan", "#008B8B"),
        ("DarkGoldenRod", "#B8860B"),
        ("DarkGray", "#A9A9A9"),
        ("DarkGreen", "#006400"),
        ("DarkKhaki", "#BDB76B"),
        ("DarkMagenta", "#8B008B"),
        ("DarkOliveGreen", "#556B2F"),
        ("DarkOrange", "#FF8C00"),
        ("DarkOrchid", "#9932CC"),
        ("DarkRed", "#8B0000"),
        ("DarkSalmon", "#E9967A"),
        ("DarkSeaGreen", "#8FBC8F"),
        ("DarkSlateBlue", "#483D8B"),
        ("DarkSlateGray", "#2F4F4F"),
        ("DarkTurquoise", "#00CED1"),
        ("DarkViolet", "#9400D3"),
        ("DeepPink", "#FF1493"),
        ("DeepSkyBlue", "#00BFFF"),
        ("DimGray", "#696969"),
        ("DodgerBlue", "#1E90FF"),
        ("FireBrick", "#B22222"),
        ("FloralWhite", "#FFFAF0"),
        ("ForestGreen", "#228B22"),
        ("Fuchsia", "#FF00FF"),
        ("Gainsboro", "#DCDCDC"),
        ("GhostWhite", "#F8F8FF"),
        ("Gold", "#FFD700"),
        ("GoldenRod", "#DAA520"),
        ("Gray", "#808080"),
        ("Green", "#008000"),
        ("GreenYellow", "#ADFF2F"),
        ("HoneyDew", "#F0FFF0"),
        ("HotPink", "#FF69B4"),
        ("IndianRed", "#CD5C5C"),
        ("Indigo", "#4B0082"),
        ("Ivory", "#FFFFF0"),
        ("Khaki", "#F0E68C"),
        ("Lavender", "#E6E6FA"),
        ("LavenderBlush", "#FFF0F5"),
        ("LawnGreen", "#7CFC00"),
        ("LemonChiffon", "#FFFACD"),
        ("LightBlue", "#ADD8E6"),
        ("LightCoral", "#F08080"),
        ("LightCyan", "#E0FFFF"),
        ("LightGoldenRodYellow", "#FAFAD2"),
        ("LightGray", "#D3D3D3"),
        ("LightGreen", "#90EE90"),
        ("LightPink", "#FFB6C1"),
        ("LightSalmon", "#FFA07A"),
        ("LightSeaGreen", "#20B2AA"),
        ("LightSkyBlue", "#87CEFA"),
        ("LightSlateGray", "#778899"),
        ("LightSteelBlue", "#B0C4DE"),
        ("LightYellow", "#FFFFE0"),
        ("Lime", "#00FF00"),
        ("LimeGreen", "#32CD32"),
        ("Linen", "#FAF0E6"),
        ("Magenta", "#FF00FF"),
        ("Maroon", "#800000"),
        ("MediumAquaMarine", "#66CDAA"),
        ("MediumBlue", "#0000CD"),
        ("MediumOrchid", "#BA55D3"),
        ("MediumPurple", "#9370DB"),
        ("MediumSeaGreen", "#3CB371"),
        ("MediumSlateBlue", "#7B68EE"),
        ("MediumSpringGreen", "#00FA9A"),
        ("MediumTurquoise", "#48D1CC"),
        ("MediumVioletRed", "#C71585"),
        ("MidnightBlue", "#191970"),
        ("MintCream", "#F5FFFA"),
        ("MistyRose", "#FFE4E1"),
        ("Moccasin", "#FFE4B5"),
        ("NavajoWhite", "#FFDEAD"),
        ("Navy", "#000080"),
        ("OldLace", "#FDF5E6"),
        ("Olive", "#808000"),
        ("OliveDrab", "#6B8E23"),
        ("Orange", "#FFA500"),
        ("OrangeRed", "#FF4500"),
        ("Orchid", "#DA70D6"),
        ("PaleGoldenRod", "#EEE8AA"),
        ("PaleGreen", "#98FB98"),
        ("PaleTurquoise", "#AFEEEE"),
        ("PaleVioletRed", "#DB7093"),
        ("PapayaWhip", "#FFEFD5"),
        ("PeachPuff", "#FFDAB9"),
        ("Peru", "#CD853F"),
        ("Pink", "#FFC0CB"),
        ("Plum", "#DDA0DD"),
        ("PowderBlue", "#B0E0E6"),
        ("Purple", "#800080"),
        ("Red", "#FF0000"),
        ("RosyBrown", "#BC8F8F"),
        ("RoyalBlue", "#4169E1"),
        ("SaddleBrown", "#8B4513"),
        ("Salmon", "#FA8072"),
        ("SandyBrown", "#F4A460"),
        ("SeaGreen", "#2E8B57"),
        ("SeaShell", "#FFF5EE"),
        ("Sienna", "#A0522D"),
        ("Silver", "#C0C0C0"),
        ("SkyBlue", "#87CEEB"),
        ("SlateBlue", "#6A5ACD"),
        ("SlateGray", "#708090"),
        ("Snow", "#FFFAFA"),
        ("SpringGreen", "#00FF7F"),
        ("SteelBlue", "#4682B4"),
        ("Tan", "#D2B48C"),
        ("Teal", "#008080"),
        ("Thistle", "#D8BFD8"),
        ("Tomato", "#FF6347"),
        ("Turquoise", "#40E0D0"),
        ("Violet", "#EE82EE"),
        ("Wheat", "#F5DEB3"),
        ("White", "#FFFFFF"),
        ("WhiteSmoke", "#F5F5F5"),
        ("Yellow", "#FFFF00"),
        ("YellowGreen", "#9ACD32"),
    ))

    safe_colors = (
        'black', 'maroon', 'green', 'navy', 'olive',
        'purple', 'teal', 'lime', 'blue', 'silver',
        'gray', 'yellow', 'fuchsia', 'aqua', 'white',
    )

    def color_name(self):
        return self.random_element(self.all_colors.keys())

    def safe_color_name(self):
        return self.random_element(self.safe_colors)

    def hex_color(self):
        return "#{}".format(
            ("%x" %
             self.random_int(
                 1, 16777215)).ljust(
                6, '0'))

    def safe_hex_color(self):
        color = ("%x" % self.random_int(0, 255)).ljust(3, '0')
        return "#{0}{0}{1}{1}{2}{2}".format(*color)

    def rgb_color(self):
        """Generate random color formatted as comma-separated RGB value

        :sample:
        """
        return ','.join(map(str, (self.random_int(0, 255) for _ in range(3))))

    def rgb_css_color(self):
        """Generate random color formatted as a CSS rgb() function

        :sample:
        """
        return 'rgb(%s)' % ','.join(
            map(str, (self.random_int(0, 255) for _ in range(3))))

    def color(self, hue=None, luminosity=None, color_format='hex'):
        """Generate random color in the specified ``color_format`` for the given ``hue`` and ``luminosity``.

        Under the hood, this method first creates a color represented in the HSV color model and then converts
        it to the desired ``color_format``. The argument ``hue`` controls the H value according to the following
        rules:

        - If the value is a number from ``0`` to ``360``, it will serve as the H value of the generated color.
        - If the value is a tuple/list of 2 numbers from 0 to 360, the color's H value will be randomly selected
          from that range.
        - If the value is a valid string, the color's H value will be randomly selected from the H range
          corresponding to the supplied string. Valid values are ``'monochrome'``, ``'red'``, ``'orange'``,
          ``'yellow'``, ``'green'``, ``'blue'``, ``'purple'``, and ``'pink'``.

        The argument ``luminosity`` influences both S and V values and is partially affected by ``hue`` as well.
        The finer details of this relationship are somewhat involved, so please refer to the source code instead
        if you wish to dig deeper. To keep the interface simple, this argument either can be omitted or can accept
        the following string values:``'bright'``, ``'dark'``, ``'light'``, or ``'random'``.

        The argument ``color_format`` controls in which color model the color is represented. Valid values are
        ``'hsv'``, ``'hsl'``, ``'rgb'``, or ``'hex'`` (default).

        :sample: hue='red'
        :sample: luminosity='light'
        :sample: hue=(100, 200), color_format='rgb'
        :sample: hue='orange', luminosity='bright'
        :sample: hue=135, luminosity='dark', color_format='hsv'
        :sample: hue=(300, 20), luminosity='random', color_format='hsl'
        """
        return RandomColor(self.generator).generate(
            hue=hue, luminosity=luminosity, color_format=color_format,
        )



import operator

from collections import Counter
from functools import reduce


def add_dicts(*args):
    """
    Adds two or more dicts together. Common keys will have their values added.

    For example::

        >>> t1 = {'a':1, 'b':2}
        >>> t2 = {'b':1, 'c':3}
        >>> t3 = {'d':4}

        >>> add_dicts(t1, t2, t3)
        {'a': 1, 'c': 3, 'b': 3, 'd': 4}

    """

    counters = [Counter(arg) for arg in args]
    return dict(reduce(operator.add, counters))



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    # conforming to http://nl.wikipedia.org/wiki/Achternaam#Naamswijziging and
    # http://en.wikipedia.org/wiki/Dutch_name#Dutch_naming_law_.28surnames.29
    # by adding a "-" between the two last names when someone is married
    formats = (
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}-{{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}-{{last_name}}',
    )

    first_names_male = (
        'Aaron', 'Abel', 'Adam', 'Aiden', 'Alex', 'Alexander', 'Ali', 'Amin',
        'Amir', 'Arie', 'Aron', 'Arthur', 'Ayden', 'Ayoub', 'Bart', 'Bas',
        'Bastiaan', 'Beau', 'Ben', 'Benjamin', 'Berat', 'Berend', 'Bilal',
        'Bjorn', 'Boaz', 'Boris', 'Bradley', 'Bram', 'Brent', 'Brian', 'Bryan',
        'Cas', 'Casper', 'Chris', 'Colin', 'Collin', 'Cornelis', 'Daan',
        'Damian', 'Dani', 'Daniel', 'Daniël', 'Dave', 'David', 'Dean', 'Dex',
        'Dion', 'Dirk', 'Duuk', 'Dylan', 'Dylano', 'Elias', 'Emir', 'Faas',
        'Fabian', 'Fedde', 'Felix', 'Finn', 'Florian', 'Floris', 'Gerrit',
        'Giel', 'Gijs', 'Giovanni', 'Guus', 'Hamza', 'Hendrik', 'Hidde',
        'Hugo', 'Ian', 'Ibrahim', 'Imran', 'Ivan', 'Jack', 'Jacob', 'Jake',
        'James', 'Jamie', 'Jan', 'Jari', 'Jason', 'Jasper', 'Jay', 'Jayden',
        'Jayson', 'Jelle', 'Jelte', 'Jens', 'Jesper', 'Jesse', 'Jim', 'Jip',
        'Job', 'Joep', 'Joey', 'Johannes', 'Jonas', 'Jonathan', 'Joost',
        'Jordy', 'Joris', 'Jorn', 'Jort', 'Joshua', 'Joël', 'Jules', 'Julian',
        'Julius', 'Jurre', 'Justin', 'Kai', 'Kay', 'Keano', 'Kevin', 'Kian',
        'Kick', 'Koen', 'Kyan', 'Kyano', 'Lars', 'Laurens', 'Lenn', 'Leon',
        'Levi', 'Lex', 'Liam', 'Loek', 'Lorenzo', 'Luc', 'Luca', 'Lucas',
        'Luka', 'Lukas', 'Luke', 'Luuk', 'Maarten', 'Mads', 'Marijn',
        'Marinus', 'Mark', 'Mart', 'Mason', 'Mathijs', 'Mats', 'Matthias',
        'Matthijs', 'Maurits', 'Max', 'Maxim', 'Mees', 'Mehmet', 'Melle',
        'Merijn', 'Micha', 'Michael', 'Mick', 'Mika', 'Mike', 'Milan', 'Milo',
        'Mohamed', 'Mohammed', 'Morris', 'Muhammed', 'Mustafa', 'Nathan',
        'Naud', 'Nick', 'Niek', 'Niels', 'Noah', 'Noud', 'Nout', 'Olaf',
        'Olivier', 'Oscar', 'Owen', 'Pepijn', 'Philip', 'Pieter', 'Pim',
        'Quinn', 'Quinten', 'Raf', 'Rafael', 'Ravi', 'Rayan', 'Rens', 'Rick',
        'Rik', 'Riley', 'Roan', 'Robin', 'Rowan', 'Ruben', 'Ryan', 'Sam',
        'Sami', 'Samuel', 'Sander', 'Sebastiaan', 'Sem', 'Senn', 'Senna',
        'Sep', 'Sepp', 'Seth', 'Siem', 'Sil', 'Simon', 'Sjoerd', 'Stan',
        'Stef', 'Stefan', 'Sten', 'Stijn', 'Sven', 'Teun', 'Thijmen', 'Thijn',
        'Thijs', 'Thom', 'Thomas', 'Ties', 'Tijmen', 'Tijn', 'Tijs', 'Tim',
        'Timo', 'Tobias', 'Tom', 'Tristan', 'Twan', 'Tycho', 'Tygo', 'Tyler',
        'Valentijn', 'Victor', 'Vigo', 'Vince', 'Vincent', 'Wesley', 'Wessel',
        'Willem', 'Wout', 'Wouter', 'Xavi', 'Yassin', 'Youssef', 'Yusuf',
        'Zakaria',
    )

    first_names_female = (
        'Aaliyah', 'Adriana', 'Aimée', 'Alicia', 'Alyssa', 'Amber', 'Amelia',
        'Amina', 'Amira', 'Amy', 'Amélie', 'Angelina', 'Anna', 'Annabel',
        'Anne', 'Annemijn', 'Anouk', 'Ashley', 'Aya', 'Aylin', 'Azra', 'Bente',
        'Benthe', 'Bibi', 'Bo', 'Britt', 'Carlijn', 'Catharina', 'Cato',
        'Ceylin', 'Charlotte', 'Chloé', 'Chloë', 'Cornelia', 'Dana', 'Danique',
        'Daphne', 'Demi', 'Dewi', 'Dina', 'Ecrin', 'Elena', 'Elif', 'Elin',
        'Eline', 'Elisa', 'Elisabeth', 'Elise', 'Eliza', 'Elizabeth', 'Elize',
        'Ella', 'Emily', 'Emma', 'Esila', 'Esmee', 'Esmée', 'Esther', 'Eva',
        'Evelien', 'Evi', 'Evie', 'Evy', 'Fabiënne', 'Fatima', 'Fay', 'Faye',
        'Feline', 'Fem', 'Femke', 'Fenna', 'Fenne', 'Fien', 'Fiene', 'Fleur',
        'Floor', 'Floortje', 'Frederique', 'Féline', 'Guusje', 'Hailey',
        'Hanna', 'Hannah', 'Helena', 'Ilse', 'Imke', 'Inaya', 'Indy', 'Iris',
        'Isa', 'Isabel', 'Isabella', 'Isabelle', 'Ise', 'Isis', 'Ivy', 'Ize',
        'Jade', 'Janna', 'Janne', 'Jasmijn', 'Jayda', 'Jaylinn', 'Jenna',
        'Jennifer', 'Jente', 'Jet', 'Jill', 'Jinthe', 'Johanna', 'Jolie',
        'Jolijn', 'Josephine', 'Joy', 'Joëlle', 'Julia', 'Julie', 'Juliette',
        'Juul', 'Karlijn', 'Kate', 'Kaylee', 'Kayleigh', 'Kiki', 'Kim',
        'Kyara', 'Kyra', 'Lana', 'Lara', 'Laura', 'Lauren', 'Leah', 'Lena',
        'Lieke', 'Lieve', 'Lily', 'Lina', 'Linde', 'Lindsey', 'Linn', 'Lisa',
        'Lisanne', 'Lise', 'Liv', 'Livia', 'Liz', 'Liza', 'Lize', 'Lizz',
        'Lizzy', 'Loes', 'Lois', 'Lola', 'Lot', 'Lotte', 'Louise', 'Loïs',
        'Lucy', 'Luna', 'Lynn', 'Maaike', 'Maartje', 'Madelief', 'Maja',
        'Mara', 'Mare', 'Maria', 'Marit', 'Maryam', 'Maud', 'Maya', 'Megan',
        'Meike', 'Melissa', 'Merel', 'Merle', 'Mette', 'Mia', 'Michelle',
        'Mila', 'Milou', 'Mirte', 'Mirthe', 'Myrthe', 'Nadia', 'Nadine',
        'Naomi', 'Nienke', 'Nikki', 'Nina', 'Ninthe', 'Nisa', 'Noa', 'Noor',
        'Noortje', 'Nora', 'Norah', 'Nova', 'Noëlle', 'Nynke', 'Olivia',
        'Phileine', 'Pien', 'Pippa', 'Pleun', 'Puck', 'Puk', 'Quinty',
        'Renske', 'Robin', 'Romy', 'Roos', 'Rosa', 'Rosalie', 'Saar', 'Sam',
        'Sanne', 'Sara', 'Sarah', 'Selena', 'Selina', 'Senna', 'Sienna',
        'Silke', 'Sofia', 'Sofie', 'Sophia', 'Sophie', 'Stella', 'Sterre',
        'Suus', 'Suze', 'Sylvie', 'Tara', 'Tess', 'Tessa', 'Tirza', 'Vajèn',
        'Valerie', 'Veerle', 'Vera', 'Victoria', 'Yara', 'Yasmin', 'Yasmine',
        'Yfke', 'Yinthe', 'Zara', 'Zeynep', 'Zoey', 'Zoë',
    )

    first_names = first_names_male + first_names_female

    last_names = (
        "'s Gravensande", 'Aalts', 'Aarden', 'Aarts', 'Adelaar', 'Adriaansen',
        'Adriaensdr', 'Adriaense', 'Adryaens', 'Aeije', 'Aelftrud van Wessex',
        'Aertsz', 'Alpaidis', 'Amalrada', 'Ansems', 'Appelman', 'Arens',
        'Arent', 'Ariens', 'Ariens Ansems', 'Arnold', 'Arts', 'Aschman',
        'Backer', 'Bakker', 'Barents', 'Bartels', 'Bastiaanse', 'Bastiaense',
        'Bave', 'Becht', 'Beekman', 'Beernink', 'Beijring', 'Bekbergen',
        'Bellemans', 'Belpere', 'Beourgeois', 'Berends', 'Berendse',
        'Bernaards', 'Bertho', 'Bezemer', 'Bierstraten', 'Bijlsma', 'Billung',
        'Blaak', 'Blees', 'Bleijenberg', 'Blewanus', 'Bloemendaal', 'Blokland',
        'Blom', 'Blom', 'Blonk', 'Boddaugh', 'Boer', 'Boer', 'Boers', 'Boeser',
        'Boetet', 'Bolkesteijn', 'Booden', 'Boogaerts', 'Borman', 'Bos', 'Bos',
        'Bosch', 'Bosch', 'Bosman', 'Boudewijns', 'Bouhuizen',
        'Bourgondië, van', 'Bouthoorn', 'Bouwhuisen', 'Brandon', 'Brands',
        'Brandt', 'Bresse', 'Bresé', 'Breugelensis', 'Briere', 'Brievingh',
        'Brisee', 'Brizee', 'Broeckx', 'Broeders', 'Broek', 'Broekhoven',
        'Broeshart', 'Bronder', 'Brouwer', 'Brouwer', 'Bruggeman', 'Brugman',
        'Bruijne van der Veen', 'Brumleve', 'Bruynzeels', 'Bud', 'Buijs',
        'Butselaar', 'Bökenkamp', 'Cadefau', 'Cammel', 'Cant', 'Carnotte',
        'Charon', 'Chevresson', 'Chotzen', 'Chrodtrud', 'Claassen', 'Claesdr',
        'Claesner', 'Coenen', 'Coolen', 'Coret', 'Coret-Coredo',
        'Coreth von und zu Coredo und Starkenberg', 'Cornelisse',
        'Cornelissen', 'Cornelisz', 'Corstiaens', 'Cosman', 'Courtier',
        'Dachgelder', 'Dachgeldt', 'Dachgelt', 'David', 'Dekker', 'Dekker',
        'Demmendaal', 'Dennenberg', 'Die Bont', 'Diesbergen', 'Dijkman',
        'Dijkstra', 'Dircken', 'Dirksen', 'Dirven', 'Doesburg', 'Doorhof',
        'Doornhem', 'Dorsman', 'Doyle', 'Draaisma', 'Dries', 'Driessen',
        'Drysdale', 'Dubois', 'Duivenvoorden', 'Eckhardt', 'Eelman', 'Eerden',
        'Ehlert', 'Eijkelboom', 'Elberts', 'Elbertse', 'Ellis', 'Elsemulder',
        'Elsenaar', 'Emmen', 'Engels', 'Erhout', 'Ernst', 'Estey', 'Everde',
        'Evers', 'Everts', 'Fechant', 'Feenstra', 'Feltzer', 'Ferran', 'Fiere',
        'Flink', 'Fortuyn', 'Frankhuizen', 'François', 'Françoise', 'Fredriks',
        'Fremie', 'Frerichs', 'Freshour', 'Friehus', 'Furda', 'Galenzone',
        'Galijn', 'Garret', 'Geerling', 'Geerts', 'Geertsen', 'Geldens',
        'Gellemeyer', 'Gemen', 'Geneart', 'Genefaas', 'Gepa van Bourgondië',
        'Gerrits', 'Gerritse', 'Gerritsen', 'Gervais', 'Ghoerle', 'Giselmeyer',
        'Glasses', 'Gnodde', 'Goderts', 'Godfrey van Alemannië', 'Goedhart',
        'Goudriaan', 'Govarts', 'Goyaerts van Waderle', 'Greij', 'Groen',
        'Groenendaal', 'Groenestein', 'Grondel', 'Groote', 'Gruijl', 'Guit',
        'Haack', 'Haengreve', 'Hagendoorn', 'Hak', 'Hakker', 'Haneberg',
        'Hanegraaff', 'Haring', 'Haselaar', 'Hazenveld', 'Heere', 'Heerkens',
        'Heerschop', 'Hehl', 'Heijman', 'Heijmans', 'Heijmen', 'Heinrichs',
        'Hekker', 'Hellevoort', 'Helmerhorst', 'Hemma van Allemanië',
        'Hendricks', 'Hendriks', 'Hendriks', 'Hendrikse',
        'Henric van den Nuwenhuse', 'Heribert van Laon', 'Hermans', 'Hermans',
        'Hexspoor', 'Heymans', 'Heyne', 'Hoedemakers', 'Hoeks', 'Hoekstra',
        'Hoelen', 'Hoes', 'Hofman', 'Hollander', 'Holthuis', 'Hondeveld',
        'Honing', 'Hoogers', 'Hoppenbrouwer', 'Horrocks', 'Houdijk', 'Huberts',
        'Huel', 'Huijben', 'Huijbrechts', 'Huijs', 'Huijzing', 'Huisman',
        'Huisman', 'Huls', 'Hulshouts', 'Hulskes', 'Hulst', 'Huurdeman',
        'Höning', 'Jaceps', 'Jacobi', 'Jacobs', 'Jacobs', 'Jacquot', 'Jans',
        'Jansdr', 'Janse', 'Jansen', 'Jansen', 'Jansen', 'Jansse', 'Janssen',
        'Janssen', 'Janssens', 'Jasperdr.', 'Jdotte', 'Jeggij', 'Jekel',
        'Jerusalem', 'Jochems', 'Jones', 'Jonker', 'Jonkman', 'Joosten',
        'Jorlink', 'Jorrisen', 'Jurrijens', 'Kallen', 'Kalman', 'Kamp',
        'Kamper', 'Karels', 'Kas', 'Kathagen', 'Keijser', 'Keijzer',
        'Keltenie', 'Kerkhof', 'Ketel', 'Ketting', 'Kirpenstein', 'Kisman',
        'Kleibrink', 'Kleijse', 'Klein', 'Klerks', 'Kleybrink', 'Klomp Jan',
        'Kloppert', 'Knoers', 'Knuf', 'Koeman', 'Kof', 'Kok', 'Kok', 'Kolen',
        'Kolster', 'Koning', 'Konings', 'Koret', 'Korsman', 'Korstman', 'Kort',
        'Kortman', 'Kosten', 'Koster', 'Koster', 'Krabbe', 'Kramer', 'Kremer',
        'Kriens', 'Kronenberg', 'Kruns', 'Kuijpers', 'Kuijpers', 'Kuilenburg',
        'Kuiper', 'Kuipers', 'Kuit', 'Kunen', 'Kwaadland', 'Köster', 'Labado',
        'Laffray', 'Lafleur', 'Lage', 'Lagerweij', 'Lambers', 'Lambregt',
        'Lamore', 'Lamotte', 'Langevoort', 'Lankle', 'Lansink', 'Lathrope',
        'Latier', 'Le Grand', 'Le Marec', 'Leene', 'Leguit', 'Lelijveld',
        'Lemmens', 'Lensen', 'Lether', 'Levesque', 'Lieshout', 'Ligtvoet',
        'Lijn', 'Lind', 'Linschoten', 'Lips', 'Loep', 'Lommert', 'Lonen',
        'Loreal', 'Lorreijn', 'Louws', 'Luboch', 'Lucas',
        'Luitgardis van Neustrië', 'Luster', 'Lutterveld', 'Maas', 'Maas',
        'Maaswinkel', 'Mahieu', 'Mallien', 'Mangel', 'Manne', 'Mansveld',
        'Mansvelt', 'Marceron', 'Marchal', 'Marchand', 'Martel', 'Martens',
        'Martens', 'Massa', 'Mater', 'Mathieu', 'Mathol', 'Mathurin',
        'Matthews', 'Meeres', 'Meeusen', 'Meijer', 'Meijer', 'Meis', 'Melet',
        'Mens', 'Mercks', 'Merckx', 'Merkx', 'Meyer', 'Meyer', 'Michiels',
        'Michielsen', 'Middelkoop', 'Mijsberg', 'Miltenburg', 'Miner',
        'Moenen', 'Moensendijk', 'Moet', 'Mol', 'Mol', 'Molegraaf', 'Molen',
        'Molenaar', 'Momberg', 'Mosley', 'Mudden', 'Muijs', 'Mulder', 'Mulder',
        'Mulders', 'Muller', 'Nedermeijer', 'Nek', 'Neuteboom', 'Neuzerling',
        'Niermann', 'Nieuwstraten', 'Nihoe', 'Nijman', 'Nollee', 'Noordijk',
        'Oda', 'Oemencs', 'Oennen', 'Olthof', 'Olykan', 'Ooms', 'Oosterhek',
        'Oosterhout', 'Oostveen', 'Opmans', 'Osterhoudt', 'Otte', 'Otto',
        'Oude Heer', 'Ouwel', 'Ouwerkerk', 'Overdijk', 'Overeem', 'Oversteeg',
        'Paillet', 'Palman', 'Pasman', 'Passchiers', 'Pastoors', 'Pauwels',
        'Peeters', 'Perck', 'Perkins', 'Peronne', 'Perrono', 'Persijn',
        'Peters', 'Peterse', 'Phillipsen', 'Pierson', 'Pieters',
        'Pieters van der Maes', 'Pison', 'Poncelet', 'Ponci', 'Pons', 'Post',
        'Post', 'Postma', 'Potters', 'Pratt', 'Prins', 'Prinsen', 'Puig',
        'Rackham', 'Rademaker', 'Ramaker', 'Recer', 'Recers', 'Rehorst',
        'Reijers', 'Reimes', 'Rek', 'Remmers', 'Ridder', 'Riem', 'Rietveld',
        'Rijcken', 'Rijks', 'Rijn', 'Rijntjes', 'Rippey', 'Risma',
        'Robbrechts Bruijne', 'Roessink', 'Romijn', 'Roodesteijn', 'Room',
        'Roose', 'Roosenboom', 'Rotteveel', 'Roukes', 'Rousselet',
        'Rouwenhorst', 'Rouwhorst', 'Rubben', 'Ruijs', 'Rutten', 'Salet',
        'Sam', 'Sanders', 'Sanders', 'Sarneel', 'Sas', 'Saxo', 'Scardino',
        'Schagen', 'Schakelaar', 'Scharroo', 'Schatteleijn', 'Scheer',
        'Scheffers', 'Schellekens', 'Schelvis', 'Schenk', 'Schenkel',
        'Scherms', 'Schiffer', 'Schilt', 'Schipper', 'Schokman', 'Scholten',
        'Scholten', 'Schotte', 'Schouten', 'Schrant', 'Schrik', 'Schroeff',
        'Schulten', 'Schuurmans', 'Schuylenborch', 'Schwartsbach',
        'Scuylenborchs', 'Segerszoen', 'Serra', 'Sestig', 'Shupe', 'Simonis',
        'Simons', 'Sire', 'Sitters', 'Slaetsdochter', 'Slagmolen',
        'Slingerland', 'Smeets', 'Smit', 'Smit', 'Smith', 'Smits', 'Smits',
        'Soos', 'Spaan', 'Spanhaak', 'Speijer', 'Spier', 'Spies', 'Spiker',
        'Spreeuw', 'Sprong', 'Spruit', 'Spruyt', 'Stamrood', 'Stange',
        'Steenbakkers', 'Steenbeek', 'Steinmeiern', 'Sterkman', 'Stettyn',
        'Stichter', 'Stinis', 'Stoffel', 'Stoffelsz', 'Stook', 'Strijker',
        'Strik', 'Stuivenberg', 'Suijker', 'Symons', 'Takkelenburg',
        'Tammerijn', 'Tamsma', 'Terry', 'Teunissen', 'Texier', 'Thatcher',
        'The Elder', 'Thomas', 'Thout', 'Tielemans', 'Tillmanno', 'Timmerman',
        'Timmermans', 'Timmermans', 'Tins', 'Tirie', 'Totwiller', 'Tuithof',
        'Uit de Willigen', 'Uittenbosch', 'Ulrich', 'Unruoch Hunerik',
        'Uphaus', 'Uphuis', 'Uphus', 'VI', 'Vaessen', 'Vallenduuk',
        'Van Bragt', 'Vandenbergh', 'Vastenhouw', 'Veenendaal', 'Veenstra',
        'Vegt', 'Velderman', 'Veltman', 'Verbeeck', 'Verbeek', 'Verbeek',
        'Verboom', 'Verbruggen', 'Verda', 'Vergeer', 'Verhaar', 'Verhagen',
        'Verharen', 'Verheij', 'Verheuvel', 'Verhoeven', 'Verhoeven',
        'Verkade', 'Vermeulen', 'Vermeulen', 'Verschuere', 'Verschut',
        'Versluijs', 'Vertoor', 'Vertooren', 'Vervoort', 'Verwoert', 'Vial',
        'Vierdag', 'Vignon', 'Vink', 'Visser', 'Volcke', 'Voortman', 'Vos',
        'Vos', 'Vrancken', 'Waardeloo', 'Wagenvoort', 'Walsteijn', 'Walter',
        'Waltrade Walderade', 'Weeldenburg', 'Weerdenburg', 'Weijland',
        'Weijters', 'Welf', 'Wendt', 'Wensen', 'Werdes', 'Werl-Arnsberg, van',
        'West-Francië, van', 'Westerbeek', 'Westerburg', 'Westermann', 'Wever',
        'Weyland', 'Weylant', 'Wigman', 'Wijland', 'Wilcken', 'Wildschut',
        'Willems', 'Willems', 'Willems van Lier', 'Willemsen', 'Willemsen',
        'Wilmont', 'Wilson', 'Winnrich', 'Winters', 'Wipstrik', 'Wolffel',
        'Wolfsdr', 'Wolfswinkel', 'Wolters', 'Wolters', 'Wolzak', 'Wooning',
        'Woudenberg', 'Wouters', 'Wouters van Eijndhoven', 'Woutersz',
        'Wright', 'Wunderink', 'Wutke', 'Zaal', 'Zeemans', 'Zeldenrust',
        'Zevenboom', 'Zijlemans', 'Zijlmans', 'Zuidweg', 'Zuijdveld', 'Zwart',
        'Zwijsen', "d' Heripon", 'de Backer', 'de Beer', 'de Bock', 'de Boer',
        'de Boer', 'de Bont', 'de Bruijn', 'de Bruijn', 'de Bruin', 'de Bruin',
        'de Bruyn', 'de Graaf', 'de Graaf', 'de Gratie', 'de Groot',
        'de Groot', 'de Grote', 'de Gruijl', 'de Gruijter', 'de Gruil',
        'de Grunt', 'de Gruson', 'de Haan', 'de Haas', 'de Heer', 'de Hoog',
        'de Hoogh', 'de Jager', 'de Jode Vastraedsd', 'de Jong', 'de Jong',
        'de Jonge', 'de Kale', 'de Keijser', 'de Keijzer', 'de Kok',
        'de Koning', 'de Koning', 'de Korte', 'de Lange', 'de Leeuw', 'de Man',
        'de Marduras', 'de Mol', 'de Nijs', 'de Pauw', 'de Plantard',
        'de Reede', 'de Roo', 'de Roos', 'de Ruiter', 'de Smit', 'de Strigter',
        'de Swart', 'de Vos', 'de Vries', 'de Vries', 'de Vroege', 'de Vrome',
        'de Werd', 'de Wit', 'de Wit', 'de la Fleche', 'den Buytelaar',
        'den Haag', 'den Teuling', 'der Kijnder', 'die Bont', 'die Pelser',
        'die Witte', 'le Briel', 'le Floch', 'le Gallen', 'le Guellec',
        'le Gulcher', 'le Luc', 'le Matelot', 'ter Waarbeek', "van 't Erve",
        "van 't Houteveen", "van 't Riet", "van 't Wel", 'van Alenburg',
        'van Allemanië', 'van Amstel', 'van Arkel', 'van Arnsberg',
        'van Asten', 'van Baalen', 'van Beaumont', 'van Beeck',
        'van Beeck Beeckmans', 'van Beek', 'van Beek', 'van Beieren',
        'van Bentheim', 'van Bergen', 'van Berkel', 'van Berkum',
        'van Bernicia', 'van Boulogne', 'van Boven', 'van Bovene',
        'van Bovenen', 'van Brenen', 'van Breugel', 'van Breukeleveen',
        'van Breukelveen', 'van Bruchem', 'van Brunswijk', 'van Bunschoten',
        'van Buuren', 'van Clootwijck', 'van Cuijck', 'van Daal',
        'van Dagsburg', 'van Dalem', 'van Dam', 'van Dam', 'van Dijk',
        'van Dijk', 'van Dillen', 'van Dokkum', 'van Dommelen', 'van Dongen',
        'van Dongen', 'van Dooren', 'van Doorn', 'van Drenthe',
        'van Duivenvoorde', 'van Duvenvoirde', 'van Duyvenvoorde', 'van Eck',
        'van Egisheim', 'van Embden', 'van Emmelen', 'van Engeland',
        'van Engelen', 'van Enschot', 'van Es', 'van Este', 'van Evelingen',
        'van Formbach', 'van Gastel', 'van Geenen', 'van Geest', 'van Geffen',
        'van Gelder', 'van Gemert', 'van Gent', 'van Ghoerle', 'van Gils',
        'van Ginkel', 'van Ginneke', 'van Goerle', 'van Gorp', 'van Grinsven',
        'van Grondelle', 'van Haarlem', 'van Haeften', 'van Hagen', 'van Ham',
        'van Hamaland', 'van Haspengouw', 'van Haspengouw Hesbaye',
        'van Hemert', 'van Henegouwen', 'van Herstal', 'van Heusden',
        'van Hoevel en van Zwindrecht', 'van Holland', 'van Hostaden',
        'van Hulten', 'van Jumiège', 'van Kasteelen', 'van Kempen',
        'van Klaarwater', 'van Kuijc', 'van Kuijc van Malsen', 'van Kusen',
        'van Laar', 'van Laarhoven', 'van Landen', 'van Laon', 'van Leeuwen',
        'van Leeuwen', 'van Leuven', 'van Liendert', 'van Limburg', 'van Loon',
        'van Loon', 'van Lucel', 'van Luin', 'van Luinenburg', 'van Luxemburg',
        'van Luyssel', 'van Maaren', 'van Maasgouw', 'van Mare', 'van Metz',
        'van Mil', 'van Mispelen', 'van Mook', 'van Munster',
        'van Nederlotharingen', 'van Nes', 'van Nimwegen', 'van Noordeloos',
        'van Noort', 'van Northeim', 'van Nus', 'van Ochten', 'van Oirschot',
        'van Olst', 'van Ommeren', 'van Ooste', 'van Oosten', 'van Oostendorp',
        'van Ooyen', 'van Opper-Lotharingen', 'van Orleans', 'van Oudewater',
        'van Parijs', 'van Poppel', 'van Praagh', 'van Rheineck', 'van Riet',
        'van Rijnsbergen', 'van Rijthoven', 'van Roijen', 'van Rooij',
        'van Rossum', 'van Saksen', 'van Salm', 'van Salmen', 'van Santen',
        'van Schevinghuizen', 'van Schweinfurt', 'van Soest', 'van Spreeuwel',
        'van Spreuwel', 'van Straaten', 'van Stralen', 'van Suinvorde',
        'van Susa', 'van Tours', 'van Tuijl', 'van Veen', 'van Velthoven',
        'van Velzen', 'van Venrooy', 'van Verdun', 'van Vermandois',
        'van Vlaanderen', 'van Vliet', 'van Voorhout', 'van Voorst',
        'van Waas', 'van Wallaert', 'van Wassenaar', 'van Wel', 'van Wessex',
        'van Westfalen', 'van Wickerode', 'van Wijk', 'van Wijland',
        'van Zwaben', 'van de Berg', 'van de Biesenbos', 'van de Biezenbos',
        'van de Brink', 'van de Coterlet', 'van de Darnau',
        'van de Eerenbeemt', 'van de Elzas', 'van de Greef',
        'van de Klashorst', 'van de Kooij', 'van de Leemput',
        'van de Noordmark', 'van de Pavert', 'van de Plas', 'van de Pol',
        'van de Veen', 'van de Velde', 'van de Velden', 'van de Ven',
        'van de Ven', 'van de Wal', 'van de Water', 'van de Weterink',
        'van de Wiel', 'van den Assem', 'van den Berg', 'van den Berg',
        'van den Bergh', 'van den Bosch', 'van den Brand', 'van den Brink',
        'van den Brink', 'van den Broek', 'van den Broek', 'van den Corput',
        'van den Eerenbeemt', 'van den Eijssel', 'van den Henst',
        'van den Heuvel', 'van den Hoek', 'van den Nieuwenhuijsen',
        'van den Nuwenhijsen', 'van den Nuwenhuijzen', 'van den Nuwenhuysen',
        'van den Nyeuwenhuysen', 'van den Oever', 'van den Pol',
        'van den Velde', 'van den Velden', 'van den Wittenboer',
        'van der Avoirt', 'van der Berg', 'van der Brink', 'van der Flaas',
        'van der Heiden', 'van der Heijden', 'van der Heijden',
        'van der Heyden', 'van der Hoeven', 'van der Horst', 'van der Horst',
        'van der Kaay', 'van der Kint', 'van der Klein', 'van der Klijn',
        'van der Laan', 'van der Laar', 'van der Laarse', 'van der Lede',
        'van der Leek', 'van der Linden', 'van der Linden', 'van der Loo',
        'van der Maath', 'van der Maes', 'van der Mast', 'van der Meer',
        'van der Meulen', 'van der Noot', 'van der Plas', 'van der Ploeg',
        'van der Pluijm', 'van der Pol', 'van der Pouw', 'van der Sande',
        'van der Schuijt', 'van der Sloot', 'van der Smeede',
        'van der Spaendonc', 'van der Spaendonck', 'van der Stael',
        'van der Stael de Jonge', 'van der Steen', 'van der Strigt',
        'van der Veen', 'van der Veiver', 'van der Velde', 'van der Velden',
        'van der Ven', 'van der Wal', 'van der Zijl', 'van het Heerenveen',
    )



from ..ar_AA import Provider as ArabicPersonProvider


class Provider(ArabicPersonProvider):
    last_names = (
        'خوري', 'شاهين', 'البرغوثي', 'الحسيني', 'الزيدانية', 'جرار',
        'السكاكيني', 'طوقان', 'نسيبة', 'النشاشيبي', 'الخازن', 'ترابين',
        'دغمش', 'دلاشة', 'الزيتاوي', 'صليبا', 'التركمان', 'الإغباري',
        'كسواني', 'جزار', 'أبو اسنينة', 'التميمي', 'الجنيدي', 'بدر',
        'أبو شقدم', 'أبو غليون', 'أبو قمر', 'الأدغم', 'حوسة', 'الصالحي',
        'حمامي', 'هاشم', 'الحنبلي', 'أستيتية', 'مرمش', 'قرادة',
        'فطاير', 'الخماش', 'الخياط', 'النعنيش', 'مهيار', 'أبو شلبك',
    )



import string
import sys
import warnings

from decimal import Decimal

from .. import BaseProvider


class Provider(BaseProvider):
    default_value_types = (
        'str', 'str', 'str', 'str', 'float', 'int', 'int', 'decimal',
        'date_time', 'uri', 'email',
    )

    def _check_signature(self, value_types, allowed_types):
        if value_types is not None and not isinstance(value_types, (list, tuple)):
            value_types = [value_types]
            warnings.warn(
                'Passing value types as positional arguments is going to be '
                'deprecated.  Pass them as a list or tuple instead.',
                PendingDeprecationWarning,
            )
        if value_types is None:
            value_types = ()
        return tuple(value_types) + allowed_types

    def pybool(self):
        return self.random_int(0, 1) == 1

    def pystr(self, min_chars=None, max_chars=20):
        """
        Generates a random string of upper and lowercase letters.
        :type min_chars: int
        :type max_chars: int
        :return: String. Random of random length between min and max characters.
        """
        if min_chars is None:
            return "".join(self.random_letters(length=max_chars))
        else:
            assert (
                max_chars >= min_chars), "Maximum length must be greater than or equal to minium length"
            return "".join(
                self.random_letters(
                    length=self.generator.random.randint(min_chars, max_chars),
                ),
            )

    def pystr_format(self, string_format='?#-###{{random_int}}{{random_letter}}', letters=string.ascii_letters):
        return self.bothify(self.generator.parse(string_format), letters=letters)

    def pyfloat(self, left_digits=None, right_digits=None, positive=False,
                min_value=None, max_value=None):
        if left_digits is not None and left_digits < 0:
            raise ValueError(
                'A float number cannot have less than 0 digits in its '
                'integer part')
        if right_digits is not None and right_digits < 0:
            raise ValueError(
                'A float number cannot have less than 0 digits in its '
                'fractional part')
        if left_digits == 0 and right_digits == 0:
            raise ValueError(
                'A float number cannot have less than 0 digits in total')
        if None not in (min_value, max_value) and min_value > max_value:
            raise ValueError('Min value cannot be greater than max value')
        if None not in (min_value, max_value) and min_value == max_value:
            raise ValueError('Min and max value cannot be the same')

        left_digits = left_digits if left_digits is not None else (
            self.random_int(1, sys.float_info.dig))
        right_digits = right_digits if right_digits is not None else (
            self.random_int(0, sys.float_info.dig - left_digits))
        sign = ''
        if (min_value is not None) or (max_value is not None):
            if max_value is not None and max_value < 0:
                max_value += 1  # as the random_int will be generated up to max_value - 1
            if min_value is not None and min_value < 0:
                min_value += 1  # as we then append digits after the left_number
            left_number = self._safe_random_int(min_value, max_value)
        else:
            sign = '+' if positive else self.random_element(('+', '-'))
            left_number = self.random_number(left_digits)

        return float("{}{}.{}".format(
            sign,
            left_number,
            self.random_number(right_digits),
        ))

    def _safe_random_int(self, min_value, max_value):
        orig_min_value = min_value
        orig_max_value = max_value

        if min_value is None:
            min_value = max_value - self.random_int()
        if max_value is None:
            max_value = min_value + self.random_int()
        if min_value == max_value:
            return self._safe_random_int(orig_min_value, orig_max_value)
        else:
            return self.random_int(min_value, max_value - 1)

    def pyint(self, min_value=0, max_value=9999, step=1):
        return self.generator.random_int(min_value, max_value, step=step)

    def pydecimal(self, left_digits=None, right_digits=None, positive=False,
                  min_value=None, max_value=None):

        float_ = self.pyfloat(
            left_digits, right_digits, positive, min_value, max_value)
        return Decimal(str(float_))

    def pytuple(self, nb_elements=10, variable_nb_elements=True, value_types=None, *allowed_types):
        return tuple(
            self.pyset(
                nb_elements,
                variable_nb_elements,
                value_types,
                *allowed_types))

    def pyset(self, nb_elements=10, variable_nb_elements=True, value_types=None, *allowed_types):
        return set(
            self._pyiterable(
                nb_elements,
                variable_nb_elements,
                value_types,
                *allowed_types))

    def pylist(self, nb_elements=10, variable_nb_elements=True, value_types=None, *allowed_types):
        return list(
            self._pyiterable(
                nb_elements,
                variable_nb_elements,
                value_types,
                *allowed_types))

    def pyiterable(
            self,
            nb_elements=10,
            variable_nb_elements=True,
            value_types=None,
            *allowed_types):
        value_types = self._check_signature(value_types, allowed_types)
        return self.random_element([self.pylist, self.pytuple, self.pyset])(
            nb_elements, variable_nb_elements, value_types, *allowed_types)

    def _random_type(self, type_list):
        value_type = self.random_element(type_list)

        method_name = "py{}".format(value_type)
        if hasattr(self, method_name):
            value_type = method_name

        return self.generator.format(value_type)

    def _pyiterable(
            self,
            nb_elements=10,
            variable_nb_elements=True,
            value_types=None,
            *allowed_types):

        value_types = self._check_signature(value_types, allowed_types)

        value_types = [t if isinstance(t, str) else getattr(t, '__name__', type(t).__name__).lower()
                       for t in value_types
                       # avoid recursion
                       if t not in ['iterable', 'list', 'tuple', 'dict', 'set']]
        if not value_types:
            value_types = self.default_value_types

        if variable_nb_elements:
            nb_elements = self.randomize_nb_elements(nb_elements, min=1)

        for _ in range(nb_elements):
            yield self._random_type(value_types)

    def pydict(self, nb_elements=10, variable_nb_elements=True, value_types=None, *allowed_types):
        """
        Returns a dictionary.

        :nb_elements: number of elements for dictionary
        :variable_nb_elements: is use variable number of elements for dictionary
        :value_types: type of dictionary values
        """
        if variable_nb_elements:
            nb_elements = self.randomize_nb_elements(nb_elements, min=1)

        return dict(zip(
            self.generator.words(nb_elements, unique=True),
            self._pyiterable(nb_elements, False, value_types, *allowed_types),
        ))

    def pystruct(self, count=10, value_types=None, *allowed_types):
        value_types = self._check_signature(value_types, allowed_types)

        value_types = [t if isinstance(t, str) else getattr(t, '__name__', type(t).__name__).lower()
                       for t in value_types
                       # avoid recursion
                       if t != 'struct']
        if not value_types:
            value_types = self.default_value_types

        types = []
        d = {}
        nd = {}
        for i in range(count):
            d[self.generator.word()] = self._random_type(value_types)
            types.append(self._random_type(value_types))
            nd[self.generator.word()] = {i: self._random_type(value_types),
                                         i + 1: [self._random_type(value_types),
                                                 self._random_type(value_types),
                                                 self._random_type(value_types)],
                                         i + 2: {i: self._random_type(value_types),
                                                 i + 1: self._random_type(value_types),
                                                 i + 2: [self._random_type(value_types),
                                                         self._random_type(value_types)]}}
        return types, d, nd
